﻿using Memory.Logic;
using System.Data.SQLite;

namespace Memory.Data
{
    public class ScoreHandler : IScoreHandler
    {
        public const string URI = "Data Source=Scores.db;Version=3;";

        public ScoreHandler()
        {
            using SQLiteConnection connection = new(URI);
            connection.Open();
            using SQLiteCommand command = new("CREATE TABLE IF NOT EXISTS Scores(ID INTEGER PRIMARY KEY, Name TEXT, Points INTEGER)", connection);
            command.ExecuteNonQuery();
            connection.Close();
        }

        public List<Score> GetTopScores()
        {
            List<Score> scores = [];
            using SQLiteConnection connection = new(URI);
            connection.Open();
            using SQLiteCommand command = new("SELECT Name, Points FROM Scores ORDER BY Points DESC LIMIT 10", connection);
            using (SQLiteDataReader reader = command.ExecuteReader())
            {
                while (reader.Read())
                {
                    scores.Add(new(reader.GetString(0), reader.GetInt32(1)));
                }
            }
            connection.Close();
            return scores;
        }

        public void WriteScore(Score score)
        {
            using SQLiteConnection connection = new(URI);
            connection.Open();
            using SQLiteCommand command = new("INSERT INTO Scores(Name, Points) VALUES(@Name, @Points)", connection);
            command.Parameters.AddWithValue("@Name", score.Name);
            command.Parameters.AddWithValue("@Points", score.Points);
            command.ExecuteNonQuery();
            connection.Close();
        }
    }
}
